from __future__ import print_function

from helper import PillowTestCase, hopper

from PIL import TiffImagePlugin, Image
from PIL.TiffImagePlugin import IFDRational

from fractions import Fraction

class Test_IFDRational(PillowTestCase):


    def _test_equal(self, num, denom, target):
        
        t = IFDRational(num, denom)

        self.assertEqual(target, t)
        self.assertEqual(t, target)
    
    def test_sanity(self):
        
        self._test_equal(1, 1, 1)
        self._test_equal(1, 1, Fraction(1,1))

        self._test_equal(2, 2, 1)
        self._test_equal(1.0, 1, Fraction(1,1))

        self._test_equal(Fraction(1,1), 1, Fraction(1,1))
        self._test_equal(IFDRational(1,1), 1, 1)
        
        
        self._test_equal(1, 2, Fraction(1,2))
        self._test_equal(1, 2, IFDRational(1,2))

    def test_nonetype(self):
        " Fails if the _delegate function doesn't return a valid function" 

        xres = IFDRational(72) 
        yres = IFDRational(72) 
        self.assert_(xres._val is not None)
        self.assert_(xres.numerator is not None)
        self.assert_(xres.denominator is not None)
        self.assert_(yres._val is not None)
        
        self.assert_(xres and 1)
        self.assert_(xres and yres)


    def test_ifd_rational_save(self):
        for libtiff in (True, False):
            TiffImagePlugin.WRITE_LIBTIFF = libtiff
            
            im = hopper()
            out = self.tempfile('temp.tiff')
            res = IFDRational(301,1)
            im.save(out, dpi=(res,res), compression='raw')

            reloaded = Image.open(out)
            self.assertEqual(float(IFDRational(301,1)),
                             float(reloaded.tag_v2[282]))




from helper import unittest, PillowTestCase, hopper
from PIL import Image

class TestImagingCoreResize(PillowTestCase):
    #see https://github.com/python-pillow/Pillow/issues/1710
    def test_overflow(self):
        im = hopper('L')
        xsize = 0x100000008 // 4
        ysize = 1000 # unimportant
        try:
            # any resampling filter will do here
            im.im.resize((xsize, ysize), Image.LINEAR) 
            self.fail("Resize should raise MemoryError on invalid xsize")
        except MemoryError:
            self.assertTrue(True, "Should raise MemoryError")

    def test_invalid_size(self):
        im = hopper()

        im.resize((100,100))
        self.assertTrue(True, "Should not Crash")
        
        try:
            im.resize((-100,100))
            self.fail("Resize should raise a value error on x negative size")
        except ValueError:
            self.assertTrue(True, "Should raise ValueError")

        try:
            im.resize((100,-100))
            self.fail("Resize should raise a value error on y negative size")
        except ValueError:
            self.assertTrue(True, "Should raise ValueError")

if __name__ == '__main__':
    unittest.main()



#
# The Python Imaging Library
# $Id$
#
# load a GIMP brush file
#
# History:
#       96-03-14 fl     Created
#
# Copyright (c) Secret Labs AB 1997.
# Copyright (c) Fredrik Lundh 1996.
#
# See the README file for information on usage and redistribution.
#

from PIL import Image, ImageFile, _binary

i32 = _binary.i32be


def _accept(prefix):
    return len(prefix) >= 8 and i32(prefix) >= 20 and i32(prefix[4:8]) == 1


##
# Image plugin for the GIMP brush format.

class GbrImageFile(ImageFile.ImageFile):

    format = "GBR"
    format_description = "GIMP brush file"

    def _open(self):

        header_size = i32(self.fp.read(4))
        version = i32(self.fp.read(4))
        if header_size < 20 or version != 1:
            raise SyntaxError("not a GIMP brush")

        width = i32(self.fp.read(4))
        height = i32(self.fp.read(4))
        color_depth = i32(self.fp.read(4))
        if width <= 0 or height <= 0 or color_depth != 1:
            raise SyntaxError("not a GIMP brush")

        comment = self.fp.read(header_size - 20)[:-1]

        self.mode = "L"
        self.size = width, height

        self.info["comment"] = comment

        # Since the brush is so small, we read the data immediately
        self.data = self.fp.read(width * height)

    def load(self):

        if not self.data:
            return

        # create an image out of the brush data block
        self.im = Image.core.new(self.mode, self.size)
        self.im.frombytes(self.data)
        self.data = b""

#
# registry

Image.register_open(GbrImageFile.format, GbrImageFile, _accept)

Image.register_extension(GbrImageFile.format, ".gbr")



from helper import unittest, PillowTestCase

from PIL import FontFile, BdfFontFile

filename = "Tests/images/courB08.bdf"


class TestFontBdf(PillowTestCase):

    def test_sanity(self):

        test_file = open(filename, "rb")
        font = BdfFontFile.BdfFontFile(test_file)

        self.assertIsInstance(font, FontFile.FontFile)
        self.assertEqual(len([_f for _f in font.glyph if _f]), 190)

    def test_invalid_file(self):
        with open("Tests/images/flower.jpg", "rb") as fp:
            self.assertRaises(SyntaxError, lambda: BdfFontFile.BdfFontFile(fp))


if __name__ == '__main__':
    unittest.main()

# End of file
